import EventEmitter from 'events';
import { writeFile } from 'fs';
import makethen from 'makethen';
const pWriteFile = makethen(writeFile);

class Element extends EventEmitter {
  constructor(params) {
    super();
    this._id = params.id;
    this._send = params.send;
  }

  async $(selector) {
    try {
      const {
        value
      } = await this._send('WebDriver:FindElement', {
        element: this._id.ELEMENT,
        value: selector,
        using: 'css selector'
      });
      return new Element({
        id: value,
        send: this._send
      });
    } catch (err) {
      if (err.message.startsWith('Unable to locate element')) {
        return null;
      }

      throw err;
    }
  }

  async $$(selector) {
    const values = await this._send('WebDriver:FindElements', {
      element: this._id.ELEMENT,
      value: selector,
      using: 'css selector'
    });
    return values.map(value => new Element({
      id: value,
      send: this._send
    }));
  }

  async focus() {
    await this._send('WebDriver:ExecuteScript', {
      'script': 'arguments[0].focus()',
      args: [this._id]
    });
  }

  async screenshot(options = {}) {
    const result = await this._send('WebDriver:TakeScreenshot', {
      id: this._id.ELEMENT,
      full: false,
      hash: false
    });
    const buffer = Buffer.from(result.value, 'base64');

    if (typeof options.path === 'string') {
      await pWriteFile(options.path, buffer, {
        encoding: null
      });
    }

    return buffer;
  }

}

export default Element;