import { Socket } from 'net';
import { createParseStream, parse, stringify } from './transport';
import FoxrError from './Error';
const CONNECTION_TIMEOUT = 10000;

const connectToMarionette = async (host, port) => {
  let globalId = 0;
  let queue = [];
  const socket = new Socket();
  await new Promise((resolve, reject) => {
    const rejectAndDestroy = error => {
      reject(error);
      socket.destroy();
    };

    socket.setTimeout(CONNECTION_TIMEOUT).once('connect', () => {
      socket.once('data', rawData => {
        const data = parse(rawData);

        if (data.applicationType === 'gecko') {
          if (data.marionetteProtocol === 3) {
            return resolve();
          }

          return rejectAndDestroy(new FoxrError('Foxr works only with Marionette protocol v3'));
        }

        rejectAndDestroy(new FoxrError('Unsupported Marionette protocol'));
      });
    }).once('timeout', () => rejectAndDestroy(new Error('Socket connection timeout'))).once('error', err => rejectAndDestroy(err)).connect(port, host);
  });
  const parseStream = createParseStream();
  parseStream.on('data', data => {
    const [type, id, error, result] = data;

    if (type === 1) {
      queue = queue.filter(item => {
        if (item.id === id) {
          if (error !== null) {
            item.reject(new FoxrError(error.message));
          } else {
            item.resolve(result);
          }

          return false;
        }

        return true;
      });
    }
  });
  socket.pipe(parseStream);
  return {
    disconnect: () => {
      socket.end();
    },
    send: (name, params = {}) => {
      return new Promise((resolve, reject) => {
        const data = stringify([0, globalId, name, params]);
        socket.write(data, 'utf8', () => {
          queue.push({
            id: globalId,
            resolve,
            reject
          });
          globalId += 1;
        });
      });
    }
  };
};

export default connectToMarionette;